# ZpCron - Cron-like scheduler for Zope
# Copyright (C) 2008  Jan Janech <janech@kios.sk>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import zLOG

from threading import Thread, Event
import datetime
import sys

class Timer(object):
    def __init__(self, callback):
        self.__callback = callback
        self.__thread = None
        self.__change = Event()
        self.__ending = Event()
        self.__lastTime = None
        self.__timeout = None
    
    def start(self):
        self.__change.clear()
        self.__thread = Thread(target = self.__run)
        self.__thread.start()
    
    def change(self):
        self.__change.set()
    
    def restart(self):
        if self.running():
            self.stop()
        self.start()
    
    def stop(self):
        self.__ending.clear()
        self.__thread = None
        self.__change.set()
        self.__ending.wait(5)
    
    def running(self):
        return self.__thread is not None and self.__thread.isAlive()
    
    def lastTime(self):
        return self.__lastTime
    
    def nextTime(self):
        if self.__lastTime is None or self.__timeout is None:
            return None
        return self.__lastTime + datetime.timedelta(0, self.__timeout)
    
    def __run(self):
        while self.__thread is not None:
            try:
                try:
                    self.__timeout = self.__callback()
                    self.__lastTime = datetime.datetime.now()
                except Exception, e:
                    zLOG.LOG('ZpCron', zLOG.ERROR, 'Exception in timer callback execution')
                    zLOG.LOG('ZpCron', zLOG.ERROR, e)
                    self.__timeout = 60
                if self.__timeout == None:
                    self.__change.wait()
                elif self.__timeout > 0:
                    self.__change.wait(self.__timeout + 10)
            except Exception, e:
                zLOG.LOG('ZpCron', zLOG.ERROR, 'Exception in timer execution')
                zLOG.LOG('ZpCron', zLOG.ERROR, e)
                self.__timeout = 60
                self.__change.wait(self.__timeout + 10)
            except:
                exc_type, exc_value, exc_tb = sys.exc_info()
                zLOG.LOG('ZpCron', zLOG.ERROR, 'UNKNOWN exception in timer execution')
                zLOG.LOG('ZpCron', zLOG.ERROR, exc_value)
                self.__timeout = 60
                self.__change.wait(self.__timeout + 10)
        self.__ending.set()
